#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Math.h>

// Arrow Key declaration
#define UP_ARROW 7
#define DOWN_ARROW 9 
#define OK_ARROW 8
#define CANCEL_ARROW 4

//Interrupt for key presses
#define INTERRUPT_PIN 3

//Ultrasonic sensor pins
#define echoPin 6 
#define trigPin 5

//Screen Dimensions
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 32

#define OLED_RESET  -1
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);


const int MPU_addr=0x68;
float axis_X,axis_Y,axis_Z;
float default_Z;

int screenno = 1;//corresponds to different menu screens


double z;

//The variables for ISR function
volatile boolean up = false;
volatile boolean down = false;
volatile boolean ok = false;
volatile boolean back = false;

boolean tryagain = true;


float default_angle;
long start;

void setup() {
  Serial.begin(9600);

  pinMode(trigPin, OUTPUT); // Sets the trigPin as an OUTPUT
  pinMode(echoPin, INPUT);// Sets the trigPin as an INPUT
  
  //Setting up all the arrow keys
  pinMode(UP_ARROW, INPUT_PULLUP);
  pinMode(DOWN_ARROW, INPUT_PULLUP);
  pinMode(OK_ARROW, INPUT_PULLUP);
  pinMode(CANCEL_ARROW, INPUT_PULLUP);
  pinMode(INTERRUPT_PIN, INPUT_PULLUP);
  
  //Attaching the Interrupt
  attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN),keycheck , FALLING);
  
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)){//0x3C is the address for 128x32 
    Serial.println("Display failed");
  }
  delay(1000);
  
  //Setting up a welcome screen
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(50,10);
  display.println("Welcome");
  display.display();
  delay(5000);
  
  //Initializing MPU6050
  Wire.begin();
  Wire.beginTransmission(MPU_addr);
  Wire.write(0x6B);
  Wire.write(0);
  Wire.endTransmission(true);
  
  display.clearDisplay();
  display.setCursor(40,10);
  display.println("Initializing....");
  display.display();
  delay(3000);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(50,10);
  display.println("Done!!");
  Serial.println("Done!!");
  display.display();
  delay(1000);

  //Calibrating part
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println("Place the unit on the ground and press OK");
  Serial.println("Place the unit on the ground and press OK");
  display.display(); 
  while(!ok){//Waiting for ok button to be pressed
    delay(500);
  }
  Serial.println("OK True");
  ok = false;
  int y =0;
  display.clearDisplay();
  display.setCursor(30,10);
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.println("Calibrating...");
  display.display();
  default_angle = getangle();
  default_Z = axis_X; // This is a crucial part in the program. Have a look in the instructable for explanation
  if(axis_X < 0)
    default_angle = 90 - default_angle;
  else
    default_angle = default_angle + 90;
  Serial.print("Angle of inclination in Z axis= ");
  Serial.print(default_angle);
  delay(500);
  Wire.endTransmission(true);
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)){//0x3C is the address for 128x32 
    Serial.println("Display failed");
  }  
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(20,10);
  display.println("Calibration Done!!");
  Serial.println("Calibration Done!!");
  display.display(); 
  delay(2000);
}

void loop() {
  while(tryagain){
  float height;  
  int j = 0;  
  Serial.println("Insideloop");
  if(screenno ==1 ){ 
      
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println(">");
  display.setCursor(10,0);
  display.println("Measure Height");
  display.setCursor(10,10);
  display.println("Measure Distance");
  display.setCursor(10,25);
  display.println("Cancel");
  display.setCursor(90,25);
  display.println("Ok");
  display.display();
  while(screenno ==1){
  while(!ok && !back && !up && !down){//Waiting for a button to be pressed
    delay(10);
  }
  if(ok){
    ok = false;
    screenno = 2;  
  }
  else if(up){
    up = false;
    if(j==1){
      display.clearDisplay();
      display.setTextSize(1);
      display.setTextColor(WHITE);
      display.setCursor(0,0);
      display.println(">");
      display.setCursor(10,0);
      display.println("Measure Height");
      display.setCursor(10,10);
      display.println("Measure Distance");
      display.setCursor(10,25);
      display.println("Cancel");
      display.setCursor(90,25);
      display.println("Ok");
      display.display();
      display.display();
      j =0;
    }
  }
  else if(down){
    down = false;
    if(j==0){
      display.clearDisplay();
      display.setTextSize(1);
      display.setTextColor(WHITE);
      display.setCursor(0,10);
      display.println(">");
      display.setCursor(10,0);
      display.println("Measure Height");
      display.setCursor(10,10);
      display.println("Measure Distance");
      display.setCursor(10,25);
      display.println("Cancel");
      display.setCursor(90,25);
      display.println("Ok");
      display.display();
      display.display();
      j =0;
      j =1;
    }
  }
  }}
    if(screenno == 2){
      int k = 0;
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,0);
  display.println("Height Mode");
  display.setCursor(0,10);
  display.println(">");
  display.setCursor(10,10);
  display.println("Automatic");
  display.setCursor(10,18);
  display.println("Manual");
  display.setCursor(10,25);
  display.println("Cancel");
  display.setCursor(90,25);
  display.println("Ok");
  display.display();
  while(screenno ==2){
  while(!ok && !back && !up && !down){//Waiting for a button to be pressed
    delay(10);
  }
  if(ok){
    ok = false;
    if(k == 0)
      screenno = 3;
    if(k == 1)
      screenno = 4;    
  }
  else if(up){
    up = false;
    if(k==1){
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,0);
  display.println("Height Mode");
  display.setCursor(0,10);
  display.println(">");
  display.setCursor(10,10);
  display.println("Automatic");
  display.setCursor(10,18);
  display.println("Manual");
  display.setCursor(10,25);
  display.println("Cancel");
  display.setCursor(90,25);
  display.println("Ok");
  display.display();
  k = 0;
    }
  }
  else if(down){
    down = false;
    if(k==0){
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,0);
  display.println("Height Mode");
  display.setCursor(0,18);
  display.println(">");
  display.setCursor(10,10);
  display.println("Automatic");
  display.setCursor(10,18);
  display.println("Manual");
  display.setCursor(10,25);
  display.println("Cancel");
  display.setCursor(90,25);
  display.println("Ok");
  display.display();
  k =1;
    }
  }
  else if(back){
    back = false;
    screenno =1;
  }
  }

  } 
    if(screenno == 3){
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(20,5);
  display.println("Height:");
  display.setCursor(60,5);
  height = getheight();
  height = height + 5;// 5 is the distance between ultrasonic sensor and accelerometer
  display.println(height);
  display.setCursor(100,5);
  display.println("cm");
  display.setTextSize(1);
  display.setCursor(10,25);
  display.println("Cancel");
  display.setCursor(90,25);
  display.println("Ok");
  
  display.display();
  while(screenno ==3){
  while(!ok && !back && !up && !down){//Waiting for a button to be pressed
    delay(10);
  }
  if(ok){
    ok = false;
    if(j == 0)
     screenno = 6;
    else if(j == 1)
      screenno = 5;     
  }
  else if(back){
    back = false;
    screenno = 2;
  }
  }

  } 
    if(screenno == 4){
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(50,0);
  display.println("Height");
  display.setCursor(45,10);
  display.println("0");
  display.setCursor(50,10);
  display.println(".");
  display.setCursor(55,10);
  display.println("0");
  display.setCursor(80,10);
  display.println("m");
  display.setCursor(10,20);
  display.println("Cancel");
  display.setCursor(80,20);
  display.println("Ok");
  display.display();
  int i =0;
  int s = 0;
  int t = 0;
  int m = 0;
  while(screenno ==4){
  while(!ok && !back && !up && !down){
    delay(10);
  }

  if(ok){
    ok = false;
    if(j == 0 && t == 2){
      if(m == -1){
        height = i + s*0.01 + 5;
        screenno = 6;}
      else if(m == 0){
        height = i + s*0.01;
        height = height*100 + 5;
        screenno = 6;}
       else if(m == 1){
        height = i + s*0.01;
        height = height*100000 + 5;
        screenno = 6;} }
    if(j == 1 && t == 2)
       if(m == -1){
        height = i + s*0.01 + 5;
        screenno = 5;}
      else if(m == 0){
        height = i + s*0.01;
        height = height*100 + 5;
        screenno = 5;}
       else if(m == 1){
        height = i + s*0.01;
        height = height*100000 + 5;
        screenno = 5;} 
    if(t == 1)
      t =2; 
    if(t == 0)
      t =1;       
  }
  else if(up){
    up = false;
    if(t == 0){
      i++;
      heightdisp(i,s,m);
    }
    else if(t == 1){
      s++;
      heightdisp(i,s,m);
    }
    else if(t == 2){
      if(m == 0){
       m =1;
       heightdisp(i,s,m);}
      else if(m == -1){
        m =1;
       heightdisp(i,s,m);  
      }
       
    }
  }
  else if(down){
    down = false;
      if(t == 0){
      if(i > 0)  
         i--;
      heightdisp(i,s,m);
    }
    else if(t == 1){
      if(s > 0)
         s--;
      heightdisp(i,s,m);
    }
      else if(t == 2){
       if(m == 1){
       m =0;
       heightdisp(i,s,m);}
      else if(m == 0){
        m =-1;
       heightdisp(i,s,m);  
      }
    }
  }
  else if(back){
    back = false;
    if(t ==0)
      screenno = 2;
    else if (t == 1)
       t = 0;  
    else if(t == 2)
      t =1;    
  }
  }
  } 
    if(screenno == 5){
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10,0);
  display.println("Point to the plane coincident part of the object and press OK");
  display.display();
  while(!ok)
    delay(10);
  ok = false;  
   display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,10);
  display.println("Calculating....");
  display.display();
  float angle = getangle();
  float height_Z = axis_Z;
  float dist;
  Serial.print(default_angle);
  Serial.print(default_Z);
  Serial.print(height_Z);
  // This is a crucial part in the program. Have a look in the instructable for explanation
  if((height_Z > 0 && default_Z < 0) || (height_Z < 0 && default_Z > 0))
     dist = height/abs(tan((180 - default_angle - angle)*PI/180));
  else
     dist = height/abs(tan((default_angle - angle)*PI/180));  
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(50,0);
  display.println("Distance");
  if(dist >= 0){  
    display.setCursor(50,10);
    display.println(dist);
    Serial.println(dist);
    display.setTextSize(1);
    display.setCursor(105,10);
    display.println("cm");
    }
  else{
    display.setCursor(50,10);
    display.println("Error");
    delay(3000);
    screenno = 7;  
  }
  display.setCursor(10,20);
  display.println("Cancel");
  display.setCursor(80,20);
  display.println("Ok");
  display.display();
  delay(2000);
  while(screenno == 5){
     while(!ok && !back && !up && !down){//Waiting for a button to be pressed
         delay(10);
         }
  if(ok){
    ok = false;
    screenno = 7;
      }
  else if(back){
    back = false;
    screenno = 5;
  }
  }
 
  } 
 if(screenno == 6){
  Serial.println(ok);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10,0);
  display.println("Point to the plane coincident part of the object and press OK");
  display.display();
  Serial.println(ok);
  while(!ok){
    delay(50);
    }
  ok =false;
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,10);
  display.println("Calculating....");
  display.display(); 
  float angle = getangle();
  float height_Z = axis_Z;
  float dist;
  Serial.println(height_Z);
  Serial.println(default_Z);
  // This is a crucial part in the program. Have a look in the instructable for explanation
  if((height_Z > 0 && default_Z < 0) || (height_Z  <0 && default_Z > 0))
     dist = height/abs(tan((180 - default_angle - angle)*PI/180));
  else
     dist = height/abs(tan((default_angle - angle)*PI/180));   
  Serial.println(dist);
   if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)){//0x3C is the address for 128x32 
    Serial.println("Display failed");
  }  
   display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10,0);
  display.println("Now hold the device vertically(at desired position) and press OK");
  display.display();
  while(!ok){
    delay(50);
    }
  ok = false;
  int h1 = getheight(); 
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10,0); 
  display.println("Now point to the top of the object and press OK");
  display.display();
  while(!ok){
    delay(50);
    }
  ok =false;
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,10);
  display.println("Calculating....");
  display.display(); 
  float topangle = getangle();
  float top_Z = axis_Z;
  Serial.println(topangle); 
   display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10,0);
  display.println("Now hold the device vertically(at desired position)  and press OK");
  display.display();
  while(!ok){
    delay(50);
    }
  ok = false;  
  int h2 = getheight(); 
  delay(500);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(10,0);
  display.println("Now point to the bottom of the object and press OK");
  display.display();
  while(!ok){
    delay(50);}
  ok =false; 
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,10);
  display.println("Calculating....");
  display.display();
 
  float bottomangle = getangle();
  float bottom_Z = axis_Z;
  Serial.println(" ");
  Serial.println(top_Z);
  Serial.println(bottom_Z);
  Serial.println(default_Z);
  float ht;
  if(top_Z >= 0 && bottom_Z >=0){
     if(default_Z <= 0)
        ht = dist*tan(abs(180 - default_angle - bottomangle)*PI/180) - dist*tan(abs(180 - default_angle - topangle)*PI/180);
     else
        ht = dist*tan(abs(default_angle - bottomangle)*PI/180) - dist*tan(abs(default_angle - topangle)*PI/180);
  }
  else if(top_Z <= 0 && bottom_Z <=0){
     if(default_Z >= 0)
        ht = dist*tan(abs(180 - default_angle - topangle)*PI/180) - dist*tan(abs(180 - default_angle - bottomangle)*PI/180);
     else
        ht = dist*tan(abs(default_angle - topangle)*PI/180) - dist*tan(abs(default_angle - bottomangle)*PI/180);
  }
  else{
    if(default_Z <= 0)
     ht = dist*tan(abs(default_angle - bottomangle)*PI/180) + dist*tan(abs(180 - default_angle - topangle)*PI/180); 
    else if(default_Z >= 0)
     ht = dist*tan(abs(180 - default_angle - bottomangle)*PI/180) + dist*tan(abs(default_angle - topangle)*PI/180);  
  }
  ht = ht + abs(h1 - h2);
  Serial.println(ht);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(20,0);
  display.println("Object Height");
  display.setCursor(50,10);
  if(ht >= 0){
    if(isnan(ht))
      display.println("0");
    else  
      display.println(ht);
    display.setCursor(105,10);
    display.println("cm");}
  else{
    display.println("Error");
    
  }  
  display.setTextSize(1);
  display.setCursor(80,20);
  display.println("Ok");
  display.display();
   while(screenno ==6){
  while(!ok && !back && !up && !down){//Waiting for a button to be pressed
    delay(10);
  }
  if(ok){
    
    ok = false;
    screenno = 7;
      }
  else if(back){
    back = false;
    screenno = 6;
  }
  }
  }  
  if(screenno == 7){
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(30,10);
  display.println("TRY AGAIN??");
  display.setTextSize(1);
  display.setCursor(10,20);
  display.println("Cancel");
  display.setCursor(80,20);
  display.println("Ok");
  display.display();
  
  while(screenno ==7){
  while(!ok && !back && !up && !down){
    delay(10);
  }
  if(ok){
    Serial.println("Last");
    ok = false;
    tryagain = true;
    screenno = 1;
    
      }
  else if(back){
    back = false;
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(30,10);
    display.println("THANK YOU!!");
    display.display();
    tryagain = false;
    screenno =10;
  }
  }      
}}}
void keycheck(){//ISR functio to check which key was pressed
  if(millis() - start >300){
    Serial.println("Clicked");
    if(digitalRead(UP_ARROW) == LOW)
       up =true;
    else if(digitalRead(DOWN_ARROW) == LOW)
       down =true;
    else if(digitalRead(OK_ARROW) == LOW)
       ok =true;
    else if(digitalRead(CANCEL_ARROW) == LOW)
       back =true;
}
  start = millis();
}

float getheight(){
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  
  // Sets the trigPin HIGH (ACTIVE) for 10 microseconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  
  // Reads the echoPin, returns the sound wave travel time in microseconds
  long  duration = pulseIn(echoPin, HIGH);
 
  // Calculating the distance
  long distance = duration * 0.034 / 2;
  
  Serial.print("Distance: ");
  Serial.print(distance);
  Serial.println(" cm");
  return distance;
}

float getangle(){
  int y =0;

  //taking accelerometer value from MPU6050
  Wire.begin();
  Wire.beginTransmission(MPU_addr);
  Wire.write(0x6B);
  Wire.write(0);
  Wire.endTransmission(true);

  //Averaging out acceleration value from 150 values
  for(int i = 0; i< 150;i++){
  Wire.beginTransmission(MPU_addr);
  Wire.write(0x3B);
  Wire.endTransmission(false);
  Wire.requestFrom(MPU_addr,6,true);
  axis_X = (Wire.read() << 8 | Wire.read()) / 16384.0; // X-axis value
  axis_Y = (Wire.read() << 8 | Wire.read()) / 16384.0; // Y-axis value
  axis_Z = (Wire.read() << 8 | Wire.read()) / 16384.0; // Z-axis value
 
  z= (atan(-1 * axis_X / sqrt(pow(axis_Y, 2) + pow(axis_Z, 2)))*180/PI) + 1.58;
  z = z + y;
  y = z;
  delay(50);
  }
  z = z/150;
  Serial.print("Angle of inclination in Z axis= ");
  Serial.println(z);
  Wire.endTransmission(true);
  delay(500);
  return z;
}
void heightdisp(int i,int s ,int m){//function for displaying height in manual mode
   display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(50,0);
  display.println("Height");
  display.setCursor(45,10);
  display.println(i);
  display.setCursor(50,10);
  display.println(".");
  display.setCursor(55,10);
  display.println(s);   
  display.setCursor(80,10);
  if(m==0)
    display.println("m");
  if(m==1)
    display.println("km"); 
  if(m==-1)
    display.println("cm");   
  display.setCursor(10,20);
  display.println("Cancel");
  display.setCursor(80,20);
  display.println("Ok");
  display.display();
}
